@@ -17,13 +17,36 @@ class Resources:
   def DownloadTempFile(self, assetType: str, path: str = '', data: str = '', dynamicAssetProviderID: str = '', noContentHeader: bool = None) -> ChiliResponse:
     return self.connector.makeRequest(
       method='get',
-      endpoint=f"resources/{assetType}/download/tempfile",
+      endpoint=f"/resources/{assetType}/download/tempfile",
       queryParams={'path':path, 'data':data, 'dynamicAssetProviderID':dynamicAssetProviderID, 'noContentHeader':noContentHeader}
     )
   def ResourceItemGetHistory(self, resourceType: str, itemID: str) -> ChiliResponse:
     return self.connector.makeRequest(
       method='get',
-      endpoint=f"resources/{resourceType}/items/{itemID}/history"
+      endpoint=f"/resources/{resourceType}/items/{itemID}/history"
+    )
+  def ResourceItemResetPreviews(self, resourceType: str, itemID: str) -> ChiliResponse:
+    return self.connector.makeRequest(
+      method='delete',
+      endpoint=f"/resources/{resourceType}/items/{itemID}/previews"
+    )
+  def ResourceItemPreviewOverride(self, resourceType: str, itemID: str) -> ChiliResponse:
+    return self.connector.makeRequest(
+      method='delete',
+      endpoint=f"/resources/{resourceType}/items/{itemID}/previewoverride"
+    )
+  def ResourceItemReplaceFile(self, resourceType: str, itemID: str, fileData: str) -> ChiliResponse:
+    return self.connector.makeRequest(
+      method="put",
+      endpoint=f"/resources/{resourceType}/items/{itemID}/file",
+      json={'fileData': fileData}
+    )
+  def ResourceItemReplaceFileWithPreviewOverride(self, resourceType: str, itemID: str, fileData: str, previewExtension: str, isPermanentPreview: bool) -> ChiliResponse:
+    return self.connector.makeRequest(
+      method="put",
+      endpoint=f"/resources/{resourceType}/items/{itemID}/filewithpreview",
+      queryParams={'previewExtension':previewExtension, 'isPermanentPreview':isPermanentPreview},
+      json={'fileData':fileData, 'previewFileData': fileData}
     )
   def ResourceItemMove(self, resourceType: str, itemID: str, newName: str, newFolderPath: str) -> ChiliResponse:
     return self.connector.makeRequest(
@@ -167,11 +190,16 @@ class Resources:
   def getDownloadURL(self, resourceType: str, itemID: str, pageNum: int = 1) -> ChiliResponse:
     return 'https://' + self.connector.baseURL + '/' + self.connector.enviroment + '/download.aspx?type=original&resourceName=' + resourceType + '&id=' + itemID + '&apiKey=' + self.connector.getAPIKey() + '&pageNum=' + pageNum
 
-  def getFullResourceTree(self, resourceType: str) -> dict:
-    items = {}
-    return self._resourceTreeIter(resourceType, items)
+  def getFullResourceTree(self, resourceType: str, folder: str = '') -> dict:
+    items = dict()
+    return self._resourceTreeIter(resourceType, items, folder=folder)
+
+  def get_items_in_tree(self, resourceType: str, folder: str = '') -> list:
+    items = list()
+    return self._resourceItemsIter(resourceType, items, folder=folder)
 
   def _resourceTreeIter(self, resourceType: str, itemsDict: dict, folder: str = '') -> dict:
+
     resp = self.ResourceGetTreeLevel(
       resourceType=resourceType,
       parentFolder=folder,
@@ -179,22 +207,41 @@ class Resources:
       includeSubDirectories=True,
       includeFiles=True
     )
-    print(resp.text)
 
     items = resp.content['tree'].get('item')
 
     if items is not None:
-      if not isinstance(items, list):
-        itemsDict[items['@name']] = items
-      else:
-        for i, v in enumerate(items):
-          if v['@isFolder'] == 'true':
-            itemsDict[v['@name']] = dict()
-            self._resourceTreeIter(resourceType, itemsDict[v['@name']], v['@path'])
-          else:
-            itemsDict[v['@name']] = v
+      if not isinstance(items, list): items = list(items)
+      for item in items:
+        if item['@isFolder'] == 'true':
+          print(f"Going in to folder {item['@name']}")
+          itemsDict[item['@name']] = dict()
+          self._resourceTreeIter(resourceType, itemsDict[item['@name']], item['@path'])
+        else:
+          itemsDict[item['@name']] = item
     return itemsDict
 
+  def _resourceItemsIter(self, resourceType: str, items_list: list = None, folder: str = '') -> list:
+
+    resp = self.ResourceGetTreeLevel(
+      resourceType=resourceType,
+      parentFolder=folder,
+      numLevels=1,
+      includeSubDirectories=True,
+      includeFiles=True
+    )
+
+    items = resp.content['tree'].get('item')
+
+    if items is not None:
+      if not isinstance(items, list): items = list(items)
+      for item in items:
+        if item['@isFolder'] == 'true':
+          self._resourceItemsIter(resourceType, items_list, item['@path'])
+        else:
+          items_list.append(item)
+    return items_list
+
 
 class Documents:
   def __init__(self, connector: ChiliConnector):
@@ -361,7 +408,6 @@ class System:
         password = login["credentials"]["Password"]
 
       requestJSON = {'userName':username, 'password': password}
-      print(requestJSON)
 
     # Request
     return self.connector.makeRequest(
